@@ -14,6 +14,13 @@
 /* If true, include GNOME support */
 #undef ENABLE_GNOME
 
+/* for gettext ... */
+#undef HAVE_LC_MESSAGES
+#undef HAVE_STPCPY
+#undef ENABLE_NLS
+#undef HAVE_CATGETS
+#undef HAVE_GETTEXT
+
 @BOTTOM@
 
 #ifdef DEBUG
