@@ -358,9 +358,22 @@ toolbar_build_children (GladeXML *xml, GtkWidget *w, GladeWidgetInfo *info,
 				if (pix) gdk_pixmap_unref(pix);
 				if (mask) gdk_bitmap_unref(mask);
 			}
-			child = gtk_toolbar_append_item(GTK_TOOLBAR(w),
-							_(label), NULL, NULL,
-							iconw, NULL, NULL);
+			if (!strcmp(cinfo->class, "GtkToggleButton"))
+				child = gtk_toolbar_append_element(
+					GTK_TOOLBAR(w),
+					GTK_TOOLBAR_CHILD_TOGGLEBUTTON, NULL,
+					_(label), NULL, NULL, iconw, NULL,
+					NULL);
+			else if (!strcmp(cinfo->class, "GtkRadioButton"))
+				child = gtk_toolbar_append_element(
+					GTK_TOOLBAR(w),
+					GTK_TOOLBAR_CHILD_RADIOBUTTON, NULL,
+					_(label), NULL, NULL, iconw, NULL,
+					NULL);
+			else
+				child = gtk_toolbar_append_item(GTK_TOOLBAR(w),
+					_(label), NULL, NULL, iconw, NULL,
+					NULL);
 			glade_xml_set_common_params(xml, child, cinfo,
 						    longname);
 		} else {
@@ -560,49 +573,14 @@ static void
 druidpage_build_children(GladeXML *xml, GtkWidget *w,
 			 GladeWidgetInfo *info, const char *longname)
 {
-	GList *tmp;
 	GladeWidgetInfo *cinfo = info->children->data;
-	GtkBox *vbox= GTK_BOX(GNOME_DRUID_PAGE_STANDARD(w)->vbox);
-	gchar *vboxname = g_strconcat(longname, ".", cinfo->name, NULL);
+	GtkWidget *vbox= GNOME_DRUID_PAGE_STANDARD(w)->vbox;
 
-	for (tmp = cinfo->children; tmp; tmp = tmp->next) {
-		GList *tmp2;
-		GtkWidget *child;
-		gboolean expand = TRUE, fill = TRUE, start = TRUE;
-		gint padding = 0;
-		cinfo = tmp->data;
-
-		child = glade_xml_build_widget(xml, cinfo, vboxname);
-		for (tmp2 = cinfo->child_attributes; tmp2; tmp2 = tmp2->next) {
-			GladeAttribute *attr = tmp2->data;
-
-			switch(attr->name[0]) {
-			case 'e':
-				if (!strcmp(attr->name, "expand"))
-					expand = attr->value[0] == 'T';
-				break;
-			case 'f':
-				if (!strcmp(attr->name, "fill"))
-					fill = attr->value[0] == 'T';
-				break;
-			case 'p':
-				if (!strcmp(attr->name, "padding"))
-					padding = strtol(attr->value, NULL, 0);
-				else if (!strcmp(attr->name, "pack"))
-					start = strcmp(attr->value,
-						       "GTK_PACK_START") == 0;
-				break;
-			}
-		}
-		if (start)
-			gtk_box_pack_start(vbox, child, expand, fill, padding);
-		else
-			gtk_box_pack_end(vbox, child, expand, fill, padding);
-	}
-	g_free(vboxname);
+	glade_xml_set_common_params(xml, vbox, cinfo, longname);
 }
 
-static void pbox_change_page(GtkWidget *child, GtkNotebook *notebook)
+static void
+pbox_change_page(GtkWidget *child, GtkNotebook *notebook)
 {
 	gint page = gtk_notebook_page_num(notebook, child);
 
@@ -704,6 +682,7 @@ static const gnome_map_t gnome_stock_pixmap_mapping [] = {
 	{ "DISABLED", GNOME_STOCK_PIXMAP_DISABLED },
 	{ "DOWN", GNOME_STOCK_PIXMAP_DOWN },
 	{ "EXEC", GNOME_STOCK_PIXMAP_EXEC },
+	{ "EXIT", GNOME_STOCK_PIXMAP_QUIT },
 	{ "FIRST", GNOME_STOCK_PIXMAP_FIRST },
 	{ "FOCUSED", GNOME_STOCK_PIXMAP_FOCUSED },
 	{ "FONT", GNOME_STOCK_PIXMAP_FONT },
@@ -733,7 +712,6 @@ static const gnome_map_t gnome_stock_pixmap_mapping [] = {
 	{ "PRINT", GNOME_STOCK_PIXMAP_PRINT },
 	{ "PROPERTIES", GNOME_STOCK_PIXMAP_PROPERTIES },
 	{ "QUIT", GNOME_STOCK_PIXMAP_QUIT },
-	{ "QUIT", GNOME_STOCK_PIXMAP_QUIT },
 	{ "REDO", GNOME_STOCK_PIXMAP_REDO },
 	{ "REFRESH", GNOME_STOCK_PIXMAP_REFRESH },
 	{ "REGULAR", GNOME_STOCK_PIXMAP_REGULAR },
@@ -788,6 +766,7 @@ static const gnome_map_t gnome_stock_menu_mapping [] = {
 	{ "CUT", GNOME_STOCK_MENU_CUT },
 	{ "DOWN", GNOME_STOCK_MENU_DOWN },
 	{ "EXEC", GNOME_STOCK_MENU_EXEC },
+	{ "EXIT", GNOME_STOCK_MENU_QUIT },
 	{ "FIRST", GNOME_STOCK_MENU_FIRST },
 	{ "FONT", GNOME_STOCK_MENU_FONT },
 	{ "FORWARD", GNOME_STOCK_MENU_FORWARD },
@@ -813,7 +792,6 @@ static const gnome_map_t gnome_stock_menu_mapping [] = {
 	{ "PRINT", GNOME_STOCK_MENU_PRINT },
 	{ "PROP", GNOME_STOCK_MENU_PROP },
 	{ "QUIT", GNOME_STOCK_MENU_QUIT },
-	{ "QUIT", GNOME_STOCK_MENU_QUIT },
 	{ "REDO", GNOME_STOCK_MENU_REDO },
 	{ "REFRESH", GNOME_STOCK_MENU_REFRESH },
 	{ "REVERT", GNOME_STOCK_MENU_REVERT },
@@ -1280,7 +1258,13 @@ clock_new(GladeXML *xml, GladeWidgetInfo *info)
 	return wid;
 }
 
-/* -- GnomeAnimator not finished */
+static GtkWidget *
+animator_new(GladeXML *xml, GladeWidgetInfo *info)
+{
+	GtkWidget *wid = gtk_type_new(gnome_animator_get_type());
+
+	return wid;
+}
 
 static GtkWidget *
 less_new(GladeXML *xml, GladeWidgetInfo *info)
@@ -1488,7 +1472,6 @@ dockitem_new(GladeXML *xml, GladeWidgetInfo *info)
 {
 	GtkWidget *wid;
 	GList *tmp;
-	char *name = NULL;
 	GnomeDockItemBehavior behaviour = GNOME_DOCK_ITEM_BEH_NORMAL;
 	GtkShadowType shadow_type = GTK_SHADOW_OUT;
 	
@@ -1507,9 +1490,7 @@ dockitem_new(GladeXML *xml, GladeWidgetInfo *info)
 					behaviour |= GNOME_DOCK_ITEM_BEH_LOCKED;
 			break;
 		case 'n':
-			if (!strcmp(attr->name, "name"))
-				name = attr->value;
-			else if (!strcmp(attr->name, "never_floating")) {
+			if (!strcmp(attr->name, "never_floating")) {
 				if (attr->value[0] == 'T')
 					behaviour |= GNOME_DOCK_ITEM_BEH_NEVER_FLOATING;
 			} else if (!strcmp(attr->name, "never_vertical")) {
@@ -1528,7 +1509,7 @@ dockitem_new(GladeXML *xml, GladeWidgetInfo *info)
 			break;
 		}
 	}
-	wid = gnome_dock_item_new(name, behaviour);
+	wid = gnome_dock_item_new(info->name, behaviour);
 	gnome_dock_item_set_shadow_type(GNOME_DOCK_ITEM(wid), shadow_type);
 	return wid;
 }
@@ -1578,6 +1559,8 @@ pixmapmenuitem_new(GladeXML *xml, GladeWidgetInfo *info)
 		GtkAccelGroup *accel;
 		guint key;
 		GtkWidget *lwid = gtk_label_new("");
+
+		gtk_misc_set_alignment(GTK_MISC(lwid), 0.0, 0.5);
 		key = gtk_label_parse_uline(GTK_LABEL(lwid), _(label));
 		if (key) {
 			accel = glade_xml_get_uline_accel(xml);
@@ -1614,7 +1597,6 @@ toolbar_new(GladeXML *xml, GladeWidgetInfo *info)
 	GtkWidget *tool;
 	GList *tmp;
 	GtkOrientation orient = GTK_ORIENTATION_HORIZONTAL;
-	GtkToolbarStyle style = GTK_TOOLBAR_BOTH;
 	GtkToolbarSpaceStyle spaces = GTK_TOOLBAR_SPACE_EMPTY;
 	int space_size = 5;
 	gboolean tooltips = TRUE;
@@ -1637,15 +1619,14 @@ toolbar_new(GladeXML *xml, GladeWidgetInfo *info)
 					attr->value);
 			break;
 		case 't':
-			if (!strcmp(attr->name, "type"))
-				style = glade_enum_from_string(
-					GTK_TYPE_TOOLBAR_STYLE, attr->value);
-			else if (!strcmp(attr->name, "tooltips"))
+			if (!strcmp(attr->name, "tooltips"))
 				tooltips = attr->value[0] == 'T';
 			break;
 		}
 	}
-	tool = gtk_toolbar_new(orient, style);
+	tool = gtk_toolbar_new(orient,
+			       gnome_preferences_get_toolbar_labels()?
+			       GTK_TOOLBAR_BOTH : GTK_TOOLBAR_ICONS);
 	gtk_toolbar_set_space_size(GTK_TOOLBAR(tool), space_size);
 	gtk_toolbar_set_space_style(GTK_TOOLBAR(tool), spaces);
 	gtk_toolbar_set_tooltips(GTK_TOOLBAR(tool), tooltips);
@@ -1681,10 +1662,9 @@ about_new(GladeXML *xml, GladeWidgetInfo *info)
 	}
 	wid = gnome_about_new(title, version, _(copyright),
 			      (const gchar **)authors, _(comments), logo);
-	/* if (title) g_free(title); */
-	/* if (version) g_free(version); */
 	if (authors) g_strfreev(authors);
 
+	glade_xml_set_window_props(GTK_WINDOW(wid), info);
 	glade_xml_set_toplevel(xml, GTK_WINDOW(wid));
 
 	return wid;
@@ -1695,23 +1675,15 @@ gnomedialog_new(GladeXML *xml, GladeWidgetInfo *info)
 {
 	GtkWidget *win;
 	GList *tmp;
-	gint xpos = -1, ypos = -1;
-	gboolean allow_shrink = TRUE, allow_grow = TRUE, auto_shrink = FALSE;
 	gboolean auto_close = FALSE, hide_on_close = FALSE;
-	gchar *title = NULL, *wmname = NULL, *wmclass = NULL;
+	gchar *title = NULL;
 
 	for (tmp = info->attributes; tmp; tmp = tmp->next) {
 		GladeAttribute *attr = tmp->data;
 
 		switch (attr->name[0]) {
 		case 'a':
-			if (!strcmp(attr->name, "allow_grow"))
-				allow_grow = attr->value[0] == 'T';
-			else if (!strcmp(attr->name, "allow_shrink"))
-				allow_shrink = attr->value[0] == 'T';
-			else if (!strcmp(attr->name, "auto_shrink"))
-				auto_shrink = attr->value[0] == 'T';
-			else if (!strcmp(attr->name, "auto_close"))
+			if (!strcmp(attr->name, "auto_close"))
 				auto_close = attr->value[0] == 'T';
 			break;
 		case 'h':
@@ -1721,33 +1693,13 @@ gnomedialog_new(GladeXML *xml, GladeWidgetInfo *info)
 			if (!strcmp(attr->name, "title"))
 				title = attr->value;
 			break;
-		case 'w':
-			if (!strcmp(attr->name, "wmclass_name"))
-				wmname = attr->value;
-			else if (!strcmp(attr->name, "wmclass_class"))
-				wmclass = attr->value;
-			break;
-		case 'x':
-			if (attr->name[1] == '\0')
-				xpos = strtol(attr->value, NULL, 0);
-			break;
-		case 'y':
-			if (attr->name[1] == '\0')
-				ypos = strtol(attr->value, NULL, 0);
-			break;
 		}
 	}
 	win = gnome_dialog_new(_(title), NULL);
-	gtk_window_set_policy(GTK_WINDOW(win), allow_shrink, allow_grow,
-			      auto_shrink);
-	if (wmname || wmclass)
-		gtk_window_set_wmclass(GTK_WINDOW(win),
-				       wmname?wmname:"", wmclass?wmclass:"");
 	gnome_dialog_set_close(GNOME_DIALOG(win), auto_close);
 	gnome_dialog_close_hides(GNOME_DIALOG(win), hide_on_close);
-	if (xpos >= 0 || ypos >= 0)
-		gtk_widget_set_uposition (win, xpos, ypos);
 
+	glade_xml_set_window_props(GTK_WINDOW(win), info);
 	glade_xml_set_toplevel(xml, GTK_WINDOW(win));
 
 	return win;
@@ -1789,6 +1741,7 @@ messagebox_new(GladeXML *xml, GladeWidgetInfo *info)
 	/* create the message box with no buttons */
 	win = gnome_message_box_new(_(message), typename, NULL);
 
+	glade_xml_set_window_props(GTK_WINDOW(win), info);
 	glade_xml_set_toplevel(xml, GTK_WINDOW(win));
 
 	return win;
@@ -1799,22 +1752,13 @@ app_new(GladeXML *xml, GladeWidgetInfo *info)
 {
 	GtkWidget *win;
 	GList *tmp;
-	gchar *title = NULL, *wmname = NULL, *wmclass = NULL;
+	gchar *title = NULL;
 	gboolean enable_layout = TRUE;
-	gboolean allow_shrink = TRUE, allow_grow = TRUE, auto_shrink = FALSE;
 
 	for (tmp = info->attributes; tmp; tmp = tmp->next) {
 		GladeAttribute *attr = tmp->data;
 
 		switch (attr->name[0]) {
-		case 'a':
-			if (!strcmp(attr->name, "allow_grow"))
-				allow_grow = attr->value[0] == 'T';
-			else if (!strcmp(attr->name, "allow_shrink"))
-				allow_shrink = attr->value[0] == 'T';
-			else if (!strcmp(attr->name, "auto_shrink"))
-				auto_shrink = attr->value[0] == 'T';
-			break;
 		case 'e':
 			if (!strcmp(attr->name, "enable_layout_config"))
 				enable_layout = attr->value[0] == 'T';
@@ -1823,22 +1767,12 @@ app_new(GladeXML *xml, GladeWidgetInfo *info)
 			if (!strcmp(attr->name, "title"))
 				title = attr->value;
 			break;
-		case 'w':
-			if (!strcmp(attr->name, "wmclass_name"))
-				wmname = attr->value;
-			else if (!strcmp(attr->name, "wmclass_class"))
-				wmclass = attr->value;
-			break;
 		}
 	}
 	win = gnome_app_new(gnome_app_id, _(title));
-	gtk_window_set_policy(GTK_WINDOW(win), allow_shrink, allow_grow,
-			      auto_shrink);
-	if (wmname || wmclass)
-		gtk_window_set_wmclass(GTK_WINDOW(win),
-				       wmname?wmname:"", wmclass?wmclass:"");
 	gnome_app_enable_layout_config(GNOME_APP(win), enable_layout);
 
+	glade_xml_set_window_props(GTK_WINDOW(win), info);
 	glade_xml_set_toplevel(xml, GTK_WINDOW(win));
 
 	return win;
@@ -1849,6 +1783,7 @@ propbox_new(GladeXML *xml, GladeWidgetInfo *info)
 {
 	GtkWidget *pbox = gnome_property_box_new();
 
+	glade_xml_set_window_props(GTK_WINDOW(pbox), info);
 	glade_xml_set_toplevel(xml, GTK_WINDOW(pbox));
 	return pbox;
 }
@@ -2011,6 +1946,7 @@ static const GladeWidgetBuildData widget_data [] = {
 	{ "GnomePixmapEntry",   pixmap_entry_new,pixmap_entry_build_children },
 	{ "GnomeDateEdit",      date_edit_new,      NULL },
 	{ "GtkClock",           clock_new,          NULL },
+	{ "GnomeAnimator",      animator_new,       NULL },
 	{ "GnomeLess",          less_new,           NULL },
 	{ "GnomeCalculator",    calculator_new,     NULL },
 	{ "GnomePaperSelector", paper_selector_new, NULL },
